/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

@import url(chrome://venkman/content/venkman-output-base.css);

.msg-data[msg-type="ERROR"]:before {
    color: orange;
}

.msg-data[msg-type="ERROR"] {
    background: darkred;
    color: white;
}

.msg-data[msg-type="ERROR"] a.venkman-link {
    color: white;
    font-weight: bold;
}
