/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

.venkman-open-tb-button {
  list-style-image: url("chrome://venkman/skin/images/step-into.png");
}
toolbar[iconsize="small"] .venkman-open-tb-button {
  list-style-image: url("chrome://venkman/skin/images/file-js-bp.png");
}
