/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Sun Microsystems code.
 *
 * The Initial Developer of the Original Code is Sun Microsystems.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Michael Buettner <michael.buettner@sun.com>
 *   Hubert Gajewski <hubert@hubertgajewski.com>, Aviary.pl
 *   Simon Vaillancourt <simon.at.orcl@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

daypicker-weekday {
    -moz-binding: url(chrome://calendar/content/calendar-daypicker.xml#daypicker-weekday);
    -moz-user-focus: normal;
}

daypicker-monthday {
    -moz-binding: url(chrome://calendar/content/calendar-daypicker.xml#daypicker-monthday);
    -moz-user-focus: normal;
}

recurrence-preview {
    -moz-binding: url(chrome://calendar/content/calendar-event-dialog-recurrence-preview.xml#recurrence-preview);
    -moz-user-focus: normal;
}

/****************************************************************************************/

attendees-list {
    -moz-binding: url(chrome://calendar/content/calendar-event-dialog-attendees.xml#attendees-list);
    -moz-user-focus: normal;
}

selection-bar {
    -moz-binding: url(chrome://calendar/content/calendar-event-dialog-attendees.xml#selection-bar);
    -moz-user-focus: normal;
}

/****************************************************************************************/

scroll-container {
    -moz-binding: url(chrome://calendar/content/calendar-event-dialog-freebusy.xml#scroll-container);
    -moz-user-focus: normal;
}

freebusy-day {
    -moz-binding: url(chrome://calendar/content/calendar-event-dialog-freebusy.xml#freebusy-day);
    -moz-user-focus: normal;
}

freebusy-timebar {
    -moz-binding: url(chrome://calendar/content/calendar-event-dialog-freebusy.xml#freebusy-timebar);
    -moz-user-focus: normal;
}

freebusy-row {
    -moz-binding: url(chrome://calendar/content/calendar-event-dialog-freebusy.xml#freebusy-row);
    -moz-user-focus: normal;
}

freebusy-grid {
    -moz-binding: url(chrome://calendar/content/calendar-event-dialog-freebusy.xml#freebusy-grid);
    -moz-user-focus: normal;
}

/****************************************************************************************/

timezone-page {
    -moz-binding: url(chrome://calendar/content/calendar-event-dialog-timezone.xml#timezone-page);
    -moz-user-focus: normal;
}
